var Button = function(pin, options) {
  options = options || {};
  this._pin = pin;

  this._normalSignal = options.normalSignal === 0 ? 0 : 1;
  this._pin.mode(this._normalSignal ? 'input_pullup' : 'input_pulldown');

  this._holdTime = options.holdTime || 1;
  this._holdTimeoutID = null;

  var debounce = options.debounce === undefined ? 20 : options.debounce;  
  setWatch(this._onChange.bind(this), this._pin, {
    repeat: true,
    edge: 'both',
    debounce: debounce
  });
};

Button.prototype.isPressed = function() {
  return this._pin.read() !== !!this._normalSignal;
};

/* Deprecated: use `isPressed` intead */
Button.prototype.read = function() {
  return this.isPressed() ? 'down' : 'up';
};

Button.prototype._onChange = function(e) {
  var pressed = this._normalSignal === 1 ? e.state : !e.state;
  var self = this;

  if (this._holdTime && pressed) {
    // emit hold event after timeout specified in options
    this._holdTimeoutID = setTimeout(function() {
      self.emit('hold');
      self._holdTimeoutID = null;
    }, this._holdTime * 1000);
  } else if (!pressed) {
    // emit click only if hold was not already emitted
    if (this._holdTimeoutID) {
      clearTimeout(this._holdTimeoutID);
      this._holdTimeoutID = null;
      this.emit('click');
    }
  }

  this.emit(pressed ? 'tap' : 'let');
};

exports.connect = function(pin, options) {
  return new Button(pin, options);
};
